@@ -82,6 +82,7 @@ request_token_page = '+request-token'
 access_token_page = '+access-token'
 authorize_token_page = '+authorize-token'
 access_token_poll_time = 1
+access_token_poll_timeout = 15 * 60
 
 EXPLOSIVE_ERRORS = (MemoryError, KeyboardInterrupt, SystemExit)
 
@@ -634,7 +635,8 @@ class AuthorizeRequestTokenWithBrowser(RequestTokenAuthorizationEngine):
         self.output(self.WAITING_FOR_USER % authorization_url)
 
         try:
-            browser = webbrowser.get().basename
+            browser_obj = webbrowser.get()
+            browser = getattr(browser_obj, "basename", None)
             console_browser = browser in self.TERMINAL_BROWSERS
         except webbrowser.Error:
             browser = None
@@ -649,8 +651,9 @@ class AuthorizeRequestTokenWithBrowser(RequestTokenAuthorizationEngine):
                 stdin.readline()
 
         self.output(self.WAITING_FOR_LAUNCHPAD)
-        if browser is not None:
+        if browser_obj is not None:
             webbrowser.open(authorization_url)
+        start_time = time.time()
         while credentials.access_token is None:
             time.sleep(access_token_poll_time)
             try:
@@ -669,6 +672,9 @@ class AuthorizeRequestTokenWithBrowser(RequestTokenAuthorizationEngine):
                     # There was an error accessing the server.
                     print("Unexpected response from Launchpad:")
                     print(e)
+            if time.time() >= start_time + access_token_poll_timeout:
+                raise TokenAuthorizationTimedOut(
+                    "Timed out after %d seconds." % access_token_poll_timeout)
 
 
 class TokenAuthorizationException(Exception):
@@ -683,6 +689,10 @@ class EndUserDeclinedAuthorization(TokenAuthorizationException):
     pass
 
 
+class TokenAuthorizationTimedOut(TokenAuthorizationException):
+    pass
+
+
 class ClientError(TokenAuthorizationException):
     pass
 
